import zmq
import threading
import uuid
from fysom import Fysom

import machinetalk.protobuf.types_pb2 as pb
from machinetalk.protobuf.message_pb2 import Container


class SimpleSubscribe(object):
    def __init__(self, debuglevel=0, debugname='Simple Subscribe'):
        self.debuglevel = debuglevel
        self.debugname = debugname
        self._error_string = ''
        self.on_error_string_changed = []
        # ZeroMQ
        context = zmq.Context()
        context.linger = 0
        self._context = context
        # pipe to signalize a shutdown
        self._shutdown = context.socket(zmq.PUSH)
        self._shutdown_uri = b'inproc://shutdown-%s' % uuid.uuid4()
        self._shutdown.bind(self._shutdown_uri)
        self._thread = None  # socket worker tread
        self._tx_lock = threading.Lock()  # lock for outgoing messages

        # Socket
        self.socket_uri = ''
        self._socket_topics = set()
        # more efficient to reuse protobuf messages
        self._socket_rx = Container()

        # callbacks
        self.on_socket_message_received = []
        self.on_state_changed = []


    @property
    def error_string(self):
        return self._error_string

    @error_string.setter
    def error_string(self, string):
        if self._error_string is string:
            return
        self._error_string = string
        for cb in self.on_error_string_changed:
            cb(string)

    def start(self):
        if self._fsm.isstate('down'):
            self._fsm.start()

    def stop(self):
        if self._fsm.isstate('up'):
            self._fsm.stop()

    def add_socket_topic(self, name):
        self._socket_topics.add(name)

    def remove_socket_topic(self, name):
        self._socket_topics.remove(name)

    def clear_socket_topics(self):
        self._socket_topics.clear()

    def _socket_worker(self, context, uri):
        poll = zmq.Poller()
        socket = context.socket(zmq.SUB)
        socket.setsockopt(zmq.LINGER, 0)
        socket.connect(uri)
        poll.register(socket, zmq.POLLIN)
        # subscribe is always connected to socket creation
        for topic in self._socket_topics:
            socket.setsockopt(zmq.SUBSCRIBE, topic)

        shutdown = context.socket(zmq.PULL)
        shutdown.connect(self._shutdown_uri)
        poll.register(shutdown, zmq.POLLIN)

        while True:
            s = dict(poll.poll())
            if shutdown in s:
                shutdown.recv()
                return  # shutdown signal
            if socket in s:
                self._socket_message_received(socket)

    def start_socket(self):
        self._thread = threading.Thread(target=self._socket_worker,
                                        args=(self._context, self.socket_uri,))
        self._thread.start()

    def stop_socket(self):
        self._shutdown.send(' ')  # trigger socket thread shutdown
        self._thread = None

    # process all messages received on socket
    def _socket_message_received(self, socket):
        (identity, msg) = socket.recv_multipart()  # identity is topic

        try:
            self._socket_rx.ParseFromString(msg)
        except DecodeError as e:
            note = 'Protobuf Decode Error: ' + str(e)
            print(note)  # TODO: decode error
            return

        if self.debuglevel > 0:
            print('[%s] received message' % self.debugname)
            if self.debuglevel > 1:
                print(self.socket_rx)
        rx = self._socket_rx

        # react to any incoming message
        if self._fsm.isstate('up'):
            self._fsm.any_msg_received()

        for cb in self.on_socket_message_received:
            cb(identity, rx)



#!/usr/bin/env python
from zeroconf import ServiceBrowser, Zeroconf
import time


class MyListener(object):

    def remove_service(self, zeroconf, type, name):
        print("Service %s removed" % (name,))

    def add_service(self, zeroconf, type, name):
        info = zeroconf.get_service_info(type, name)
        service = info.properties['service']
        uuid = info.properties['uuid']
        print('%s %s' % (service, uuid))
        # print("Service %s added, service info: %s, %s" % (name, service, uuid))


zeroconf = Zeroconf()
listener = MyListener()
browser = ServiceBrowser(zeroconf, "_machinekit._tcp.local.", listener)

try:
    while True:
        time.sleep(0.2)
except KeyboardInterrupt:
    pass
finally:
    zeroconf.close()



import pytest


@pytest.fixture
def halremote():
    from pymachinetalk import halremote
    return halremote


@pytest.fixture
def dns_sd():
    from pymachinetalk import dns_sd
    return dns_sd


def test_halremote_integration(halremote, dns_sd):
    rcomp = halremote.RemoteComponent('test')
    rcomp.newpin('foo', halremote.HAL_BIT, halremote.HAL_OUT)
    rcomp.newpin('bar', halremote.HAL_FLOAT, halremote.HAL_IN)

    sd = dns_sd.ServiceDiscovery()
    sd.register(rcomp)



import zmq
import threading
import uuid
from fysom import Fysom
from ..common.rpcclient import RpcClient
from ..common.subscribe import Subscribe
from ..common.publish import Publish

import machinetalk.protobuf.types_pb2 as pb
from machinetalk.protobuf.message_pb2 import Container


class SyncClient(object):
    def __init__(self, debuglevel=0, debugname='Sync Client'):
        self.debuglevel = debuglevel
        self.debugname = debugname
        self._error_string = ''
        self.on_error_string_changed = []

        # Sync
        self._sync_channel = RpcClient(debuglevel=debuglevel)
        self._sync_channel.debugname = '%s - %s' % (self.debugname, 'sync')
        self._sync_channel.on_state_changed.append(self._sync_channel_state_changed)
        self._sync_channel.on_socket_message_received.append(self._sync_channel_message_received)
        # more efficient to reuse protobuf messages
        self._sync_rx = Container()
        self._sync_tx = Container()

        # Sub
        self._sub_channel = Subscribe(debuglevel=debuglevel)
        self._sub_channel.debugname = '%s - %s' % (self.debugname, 'sub')
        self._sub_channel.on_state_changed.append(self._sub_channel_state_changed)
        self._sub_channel.on_socket_message_received.append(self._sub_channel_message_received)
        # more efficient to reuse protobuf messages
        self._sub_rx = Container()

        # Pub
        self._pub_channel = Publish(debuglevel=debuglevel)
        self._pub_channel.debugname = '%s - %s' % (self.debugname, 'pub')
        # more efficient to reuse protobuf messages
        self._pub_tx = Container()

        # callbacks
        self.on_sync_message_received = []
        self.on_sub_message_received = []
        self.on_state_changed = []

        # fsm
        self._fsm = Fysom({'initial': 'down',
                          'events': [
                            {'name': 'start', 'src': 'down', 'dst': 'trying'},
                            {'name': 'sync_state_up', 'src': 'trying', 'dst': 'syncing'},
                            {'name': 'stop', 'src': 'trying', 'dst': 'down'},
                            {'name': 'sync_state_trying', 'src': 'syncing', 'dst': 'trying'},
                            {'name': 'sub_state_up', 'src': 'syncing', 'dst': 'synced'},
                            {'name': 'stop', 'src': 'syncing', 'dst': 'down'},
                            {'name': 'sub_state_trying', 'src': 'synced', 'dst': 'syncing'},
                            {'name': 'sync_state_trying', 'src': 'synced', 'dst': 'trying'},
                            {'name': 'stop', 'src': 'synced', 'dst': 'down'},
                          ]})

        self._fsm.ondown = self._on_fsm_down
        self._fsm.onafterstart = self._on_fsm_start
        self._fsm.ontrying = self._on_fsm_trying
        self._fsm.onaftersync_state_up = self._on_fsm_sync_state_up
        self._fsm.onafterstop = self._on_fsm_stop
        self._fsm.onsyncing = self._on_fsm_syncing
        self._fsm.onaftersync_state_trying = self._on_fsm_sync_state_trying
        self._fsm.onaftersub_state_up = self._on_fsm_sub_state_up
        self._fsm.onsynced = self._on_fsm_synced
        self._fsm.onaftersub_state_trying = self._on_fsm_sub_state_trying

    def _on_fsm_down(self, _):
        if self.debuglevel > 0:
            print('[%s]: state DOWN' % self.debugname)
        for cb in self.on_state_changed:
            cb('down')
        return True

    def _on_fsm_start(self, _):
        if self.debuglevel > 0:
            print('[%s]: event START' % self.debugname)
        self.start_sync_channel()
        self.start_pub_channel()
        return True

    def _on_fsm_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: state TRYING' % self.debugname)
        for cb in self.on_state_changed:
            cb('trying')
        return True

    def _on_fsm_sync_state_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: event SYNC STATE UP' % self.debugname)
        self.send_sync()
        self.start_sub_channel()
        return True

    def _on_fsm_stop(self, _):
        if self.debuglevel > 0:
            print('[%s]: event STOP' % self.debugname)
        self.stop_sync_channel()
        self.stop_sub_channel()
        self.stop_pub_channel()
        return True

    def _on_fsm_syncing(self, _):
        if self.debuglevel > 0:
            print('[%s]: state SYNCING' % self.debugname)
        for cb in self.on_state_changed:
            cb('syncing')
        return True

    def _on_fsm_sync_state_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: event SYNC STATE TRYING' % self.debugname)
        self.stop_sub_channel()
        return True

    def _on_fsm_sub_state_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: event SUB STATE UP' % self.debugname)
        self.synced()
        return True

    def _on_fsm_synced(self, _):
        if self.debuglevel > 0:
            print('[%s]: state SYNCED' % self.debugname)
        for cb in self.on_state_changed:
            cb('synced')
        return True

    def _on_fsm_sub_state_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: event SUB STATE TRYING' % self.debugname)
        self.send_sync()
        return True

    @property
    def error_string(self):
        return self._error_string

    @error_string.setter
    def error_string(self, string):
        if self._error_string is string:
            return
        self._error_string = string
        for cb in self.on_error_string_changed:
            cb(string)

    @property
    def sync_uri(self):
        return self._sync_channel.socket_uri

    @sync_uri.setter
    def sync_uri(self, value):
        self._sync_channel.socket_uri = value

    @property
    def sub_uri(self):
        return self._sub_channel.socket_uri

    @sub_uri.setter
    def sub_uri(self, value):
        self._sub_channel.socket_uri = value

    @property
    def pub_uri(self):
        return self._pub_channel.socket_uri

    @pub_uri.setter
    def pub_uri(self, value):
        self._pub_channel.socket_uri = value

    @property
    def pub_port(self):
        return self._pub_channel.socket_port

    @property
    def pub_dsn(self):
        return self._pub_channel.socket_dsn

    def start(self):
        if self._fsm.isstate('down'):
            self._fsm.start()

    def stop(self):
        if self._fsm.isstate('trying'):
            self._fsm.stop()
        elif self._fsm.isstate('syncing'):
            self._fsm.stop()
        elif self._fsm.isstate('synced'):
            self._fsm.stop()

    def add_sub_topic(self, name):
        self._sub_channel.add_socket_topic(name)

    def remove_sub_topic(self, name):
        self._sub_channel.remove_socket_topic(name)

    def clear_sub_topics(self):
        self._sub_channel.clear_socket_topics()

    def add_pub_topic(self, name):
        self._pub_channel.add_socket_topic(name)

    def remove_pub_topic(self, name):
        self._pub_channel.remove_socket_topic(name)

    def clear_pub_topics(self):
        self._pub_channel.clear_socket_topics()

    def start_sync_channel(self):
        self._sync_channel.start()

    def stop_sync_channel(self):
        self._sync_channel.stop()

    def start_sub_channel(self):
        self._sub_channel.start()

    def stop_sub_channel(self):
        self._sub_channel.stop()

    def start_pub_channel(self):
        self._pub_channel.start()

    def stop_pub_channel(self):
        self._pub_channel.stop()

    # process all messages received on sync
    def _sync_channel_message_received(self, rx):

        for cb in self.on_sync_message_received:
            cb(rx)

    # process all messages received on sub
    def _sub_channel_message_received(self, identity, rx):

        for cb in self.on_sub_message_received:
            cb(identity, rx)

    def send_sync_message(self, msg_type, tx):
        self._sync_channel.send_socket_message(msg_type, tx)

    def send_pub_message(self, identity, msg_type, tx):
        self._pub_channel.send_socket_message(identity, msg_type, tx)

    def send_sync(self):
        tx = self._sync_tx
        self.send_sync_message(pb.MT_SYNC, tx)

    def send_incremental_update(self, identity, tx):
        ids = [identity]
        for receiver in ids:
            self.send_pub_message(receiver, pb.MT_INCREMENTAL_UPDATE, tx)

    def _sync_channel_state_changed(self, state):

        if (state == 'trying'):
            if self._fsm.isstate('syncing'):
                self._fsm.sync_state_trying()
            elif self._fsm.isstate('synced'):
                self._fsm.sync_state_trying()

        elif (state == 'up'):
            if self._fsm.isstate('trying'):
                self._fsm.sync_state_up()

    def _sub_channel_state_changed(self, state):

        if (state == 'trying'):
            if self._fsm.isstate('synced'):
                self._fsm.sub_state_trying()

        elif (state == 'up'):
            if self._fsm.isstate('syncing'):
                self._fsm.sub_state_up()



import os
from urlparse import urlparse
import ftplib
import threading
from dns_sd import ServiceContainer, Service

# protobuf
from common import MessageObject, recurse_descriptor, recurse_message, ComponentBase
from machinetalk.protobuf.message_pb2 import Container
import machinetalk.protobuf.types_pb2 as types
import machinetalk.protobuf.motcmds_pb2 as motcmds
from machinetalk.protobuf.status_pb2 import *
from machinetalk_core.application.statusbase import StatusBase
from machinetalk_core.application.commandbase import CommandBase
from machinetalk_core.application.errorbase import ErrorBase

ORIGIN_G54 = types.ORIGIN_G54
ORIGIN_G55 = types.ORIGIN_G55
ORIGIN_G56 = types.ORIGIN_G56
ORIGIN_G57 = types.ORIGIN_G57
ORIGIN_G58 = types.ORIGIN_G58
ORIGIN_G59 = types.ORIGIN_G59
ORIGIN_G59_1 = types.ORIGIN_G59_1
ORIGIN_G59_2 = types.ORIGIN_G59_2
ORIGIN_G59_3 = types.ORIGIN_G59_2

MOTION_UNINITIALIZED = types.UNINITIALIZED_STATUS
MOTION_DONE = types.RCS_DONE
MOTION_EXEC = types.RCS_EXEC
MOTION_ERROR = types.RCS_ERROR
MOTION_RECEIVED = types.RCS_RECEIVED

MOTION_TYPE_NONE = motcmds._EMC_MOTION_TYPE_NONE
MOTION_TYPE_TRAVERSE = motcmds._EMC_MOTION_TYPE_TRAVERSE
MOTION_TYPE_FEED = motcmds._EMC_MOTION_TYPE_FEED
MOTION_TYPE_ARC = motcmds._EMC_MOTION_TYPE_ARC
MOTION_TYPE_TOOLCHANGEE = motcmds._EMC_MOTION_TYPE_TOOLCHANGE
MOTION_TYPE_PROBING = motcmds._EMC_MOTION_TYPE_PROBING
MOTION_TYPE_INDEXROTARY = motcmds._EMC_MOTION_TYPE_INDEXROTARY

RELEASE_BRAKE = 0
ENGAGE_BRAKE = 1

JOG_STOP = 0
JOG_CONTINUOUS = 1
JOG_INCREMENT = 2

SPINDLE_FORWARD = 0
SPINDLE_REVERSE = 1
SPINDLE_OFF = 2
SPINDLE_DECREASE = 3
SPINDLE_INCREASE = 4
SPINDLE_CONSTANT = 5

NML_ERROR = types.MT_EMC_NML_ERROR
NML_TEXT = types.MT_EMC_NML_TEXT
NML_DISPLAY = types.MT_EMC_NML_DISPLAY
OPERATOR_ERROR = types.MT_EMC_OPERATOR_ERROR
OPERATOR_TEXT = types.MT_EMC_OPERATOR_TEXT
OPERATOR_DISPLAY = types.MT_EMC_OPERATOR_DISPLAY


class ApplicationStatus(ComponentBase, StatusBase, ServiceContainer):

    def __init__(self, debug=False):
        StatusBase.__init__(self, debuglevel=int(debug))
        ComponentBase.__init__(self)
        ServiceContainer.__init__(self)
        self.config_condition = threading.Condition(threading.Lock())
        self.io_condition = threading.Condition(threading.Lock())
        self.motion_condition = threading.Condition(threading.Lock())
        self.task_condition = threading.Condition(threading.Lock())
        self.interp_condition = threading.Condition(threading.Lock())
        self.synced_condition = threading.Condition(threading.Lock())
        self.debug = debug

        # callbacks
        self.on_synced_changed = []

        self.synced = False

        # status containers, also used to expose data
        self._io_data = None
        self._motion_data = None
        self._config_data = None
        self._task_data = None
        self._interp_data = None
        # required for object initialization
        self._container = Container()
        self._initialize_object('io')
        self._initialize_object('config')
        self._initialize_object('motion')
        self._initialize_object('task')
        self._initialize_object('interp')

        self._synced_channels = set()
        self.channels = set(['motion', 'config', 'task', 'io', 'interp'])

        self._status_service = Service(type_='service')
        self.add_service(self._status_service)
        self.on_services_ready_changed.append(self._on_services_ready_changed)

    def _on_services_ready_changed(self, ready):
        self.status_uri = self._status_service.uri
        self.ready = ready

    # make sure locks are used when accessing properties
    # should we return a copy instead of the reference?
    @property
    def io(self):
        with self.io_condition:
            return self._io_data

    @property
    def config(self):
        with self.config_condition:
            return self._config_data

    @property
    def motion(self):
        with self.motion_condition:
            return self._motion_data

    @property
    def task(self):
        with self.task_condition:
            return self._task_data

    @property
    def interp(self):
        with self.interp_condition:
            return self._interp_data

    def wait_synced(self, timeout=None):
        with self.synced_condition:
            if self.synced:
                return True
            self.synced_condition.wait(timeout=timeout)
            return self.synced

    def wait_config_updated(self, timeout=None):
        with self.config_condition:
            self.config_condition.wait(timeout=timeout)

    def wait_io_updated(self, timeout=None):
        with self.io_condition:
            self.io_condition.wait(timeout=timeout)

    def wait_motion_updated(self, timeout=None):
        with self.motion_condition:
            self.motion_condition.wait(timeout=timeout)

    def wait_task_updated(self, timeout=None):
        with self.task_condition:
            self.task_condition.wait(timeout=timeout)

    def wait_interp_updated(self, timeout=None):
        with self.interp_condition:
            self.interp_condition.wait(timeout=timeout)

    def emcstat_full_update_received(self, topic, rx):
        self._emcstat_update_received(topic, rx)
        self._update_synced_channels(topic)

    def emcstat_incremental_update_received(self, topic, rx):
        self._emcstat_update_received(topic, rx)

    def _emcstat_update_received(self, topic, rx):
        if topic == 'motion' and rx.HasField('emc_status_motion'):
            self._update_motion_object(rx.emc_status_motion)
        elif topic == 'config' and rx.HasField('emc_status_config'):
            self._update_config_object(rx.emc_status_config)
        elif topic == 'io' and rx.HasField('emc_status_io'):
            self._update_io_object(rx.emc_status_io)
        elif topic == 'task' and rx.HasField('emc_status_task'):
            self._update_task_object(rx.emc_status_task)
        elif topic == 'interp' and rx.HasField('emc_status_interp'):
            self._update_interp_object(rx.emc_status_interp)

    def _update_synced_channels(self, channel):
        self._synced_channels.add(channel)
        if (self._synced_channels == self.channels) and not self.synced:
            self.channels_synced()

    # slot
    def sync_status(self):
        self._update_synced(True)

    # slot
    def unsync_status(self):
        self._synced_channels.clear()
        self._update_synced(False)

    def _update_synced(self, synced):
        with self.synced_condition:
            self.synced = synced
            self.synced_condition.notify()
        for cb in self.on_synced_changed:
            cb(synced)

    # slot
    def update_topics(self):
        self.clear_status_topics()
        for channel in  self.channels:
            self.add_status_topic(channel)
            self._initialize_object(channel)

    def _initialize_object(self, channel):
        if channel == 'io':
            self._io_data = MessageObject()
            recurse_descriptor(self._container.emc_status_io.DESCRIPTOR, self._io_data)
        elif channel == 'config':
            self._config_data = MessageObject()
            recurse_descriptor(self._container.emc_status_config.DESCRIPTOR, self._config_data)
        elif channel == 'motion':
            self._motion_data = MessageObject()
            recurse_descriptor(self._container.emc_status_motion.DESCRIPTOR, self._motion_data)
        elif channel == 'task':
            self._task_data = MessageObject()
            recurse_descriptor(self._container.emc_status_task.DESCRIPTOR, self._task_data)
        elif channel == 'interp':
            self._interp_data = MessageObject()
            recurse_descriptor(self._container.emc_status_interp.DESCRIPTOR, self._interp_data)

    def _update_motion_object(self, data):
        with self.motion_condition:
            recurse_message(data, self._motion_data)
            self.motion_condition.notify()

    def _update_config_object(self, data):
        with self.config_condition:
            recurse_message(data, self._config_data)
            self.config_condition.notify()

    def _update_io_object(self, data):
        with self.io_condition:
            recurse_message(data, self._io_data)
            self.io_condition.notify()

    def _update_task_object(self, data):
        with self.task_condition:
            recurse_message(data, self._task_data)
            self._update_running()
            self.task_condition.notify()

    def _update_interp_object(self, data):
        with self.interp_condition:
            recurse_message(data, self._interp_data)
            self._update_running()
            self.interp_condition.notify()

    def _update_running(self):
        running = (self._task_data.task_mode == EMC_TASK_MODE_AUTO \
                   or self._task_data.task_mode == EMC_TASK_MODE_MDI) \
                   and self._interp_data.interp_state == EMC_TASK_INTERP_IDLE

        self.running = running


class ApplicationCommand(ComponentBase, CommandBase, ServiceContainer):

    def __init__(self, debug=False):
        CommandBase.__init__(self, debuglevel=int(debug))
        ComponentBase.__init__(self)
        ServiceContainer.__init__(self)
        self.completed_condition = threading.Condition(threading.Lock())
        self.executed_condition = threading.Condition(threading.Lock())
        self.connected_condition = threading.Condition(threading.Lock())
        self.debug = debug

        # callbacks
        self.on_connected_changed = []

        self.connected = False

        self.ticket = 1  # stores the local ticket number
        self.executed_ticket = 0  # last tick number from executed feedback
        self.completed_ticket = 0  # last tick number from executed feedback
        self._executed_updated = False
        self._completed_updated = False

        # more efficient to reuse a protobuf message
        self._tx = Container()

        self._command_service = Service(type_='command')
        self.add_service(self._command_service)
        self.on_services_ready_changed.append(self._on_services_ready_changed)

    def _on_services_ready_changed(self, ready):
        self.command_uri = self._command_service.uri
        self.ready = ready

    def emccmd_executed_received(self, rx):
        with self.executed_condition:
            self.executed_ticket = rx.reply_ticket
            self._executed_updated = True
            self.executed_condition.notify()

    def emccmd_completed_received(self, rx):
        with self.completed_condition:
            self.completed_ticket = rx.reply_ticket
            self._completed_updated = True
            self.completed_condition.notify()

    def wait_executed(self, ticket=None, timeout=None):
        with self.executed_condition:
            if ticket and ticket <= self.executed_ticket:  # very likely that we already received the reply
                return True

            while True:
                self._executed_updated = False
                self.executed_condition.wait(timeout=timeout)
                if not self._executed_updated:
                    return False  # timeout
                if ticket is None or ticket == self.executed_ticket:
                    return True

    def wait_completed(self, ticket=None, timeout=None):
        with self.completed_condition:
            if ticket and ticket < self.completed_ticket:  # very likely that we already received the reply
                return True

            while True:
                self._completed_updated = False
                self.completed_condition.wait(timeout=timeout)
                if not self._completed_updated:
                    return False  # timeout
                if ticket is None or ticket == self.completed_ticket:
                    return True

    def wait_connected(self, timeout=None):
        with self.connected_condition:
            if self.connected:
                return True
            self.connected_condition.wait(timeout=timeout)
            return self.connected

    # slot
    def set_connected(self):
        self._update_connected(True)

    # slot
    def clear_connected(self):
        self._update_connected(False)

    def _update_connected(self, connected):
        with self.connected_condition:
            self.connected = connected
            self.connected_condition.notify()
        for cb in self.on_connected_changed:
            cb(connected)

    def _take_ticket(self):
        self.ticket += 1
        return self.ticket

    def abort(self, interpreter='execute'):
        if not self.connected:
            return None

        self._tx.interp_name = interpreter
        self.send_emc_task_abort(self._tx)
        return self._take_ticket()

    def run_program(self, line_number, interpreter='execute'):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.line_number = line_number
        self._tx.interp_name = interpreter

        self.send_emc_task_plan_run(self._tx)
        return self._take_ticket()

    def pause_program(self, interpreter='execute'):
        if not self.connected:
            return None

        self._tx.interp_name = interpreter

        self.send_emc_task_plan_pause(self._tx)
        return self._take_ticket()

    def step_program(self, interpreter='execute'):
        if not self.connected:
            return None

        self._tx.interp_name = interpreter

        self.send_emc_task_plan_step(self._tx)
        return self._take_ticket()

    def resume_program(self, interpreter='execute'):
        if not self.connected:
            return None

        self._tx.interp_name = interpreter

        self.send_emc_task_plan_resume(self._tx)
        return self._take_ticket()

    def set_task_mode(self, mode, interpreter='execute'):
        if not self.connected:
            return

        params = self._tx.emc_command_params
        params.task_mode = mode
        self._tx.interp_name = interpreter

        self.send_emc_task_set_mode(self._tx)
        return self._take_ticket()

    def set_task_state(self, state, interpreter='execute'):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.task_state = state
        self._tx.interp_name = interpreter

        self.send_emc_task_set_state(self._tx)
        return self._take_ticket()

    def open_program(self, file_name, interpreter='execute'):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.path = file_name
        self._tx.interp_name = interpreter

        self.send_emc_task_plan_open(self._tx)
        return self._take_ticket()

    def reset_program(self, interpreter='execute'):
        if not self.connected:
            return None

        self._tx.interp_name = interpreter

        self.send_emc_task_plan_init(self._tx)
        return self._take_ticket()

    def execute_mdi(self, command, interpreter='execute'):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.command = command
        self._tx.interp_name = interpreter

        self.send_emc_task_plan_execute(self._tx)
        return self._take_ticket()

    def set_spindle_brake(self, brake):
        if not self.connected:
            return None

        if brake == ENGAGE_BRAKE:
            self.send_emc_spindle_brake_engage()
        elif brake == RELEASE_BRAKE:
            self.send_emc_spindle_brake_release()
        return self._take_ticket()

    def set_debug_level(self, debug_level):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.debug_level = debug_level
        self._tx.interp_name = debug_level

        self.send_emc_set_debug(self._tx)
        return self._take_ticket()

    def set_feed_override(self, scale):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.scale = scale

        self.send_emc_traj_set_scale(self._tx)
        return self._take_ticket()

    def set_flood_enabled(self, enable):
        if not self.connected:
            return None

        if enable:
            self.send_emc_coolant_flood_on(self._tx)
        else:
            self.send_emc_coolant_flood_off(self._tx)
        return self._take_ticket()

    def home_axis(self, index):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.index = index

        self.send_emc_axis_home(self._tx)
        return self._take_ticket()

    def jog(self, jog_type, axis, velocity=0.0, distance=0.0):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.index = axis

        if jog_type == JOG_STOP:
            self.send_emc_axis_abort(self._tx)
        elif jog_type == JOG_CONTINUOUS:
            params.velocity = velocity
            self.send_send_emc_axis_jog(self._tx)
        elif jog_type == JOG_INCREMENT:
            params.velocity = velocity
            params.distance = distance
            self.send_emc_axis_incr_jog(self._tx)
        else:
            self._tx.Clear()
            return None

        return self._take_ticket()

    def load_tool_table(self):
        if not self.connected:
            return None

        self.send_emc_tool_load_tool_table(self._tx)
        return self._take_ticket()

    def update_tool_table(self, tool_table):
        pass  # TODO

    def set_maximum_velocity(self, velocity):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.velocity = velocity

        self.send_emc_traj_set_max_velocity(self._tx)
        return self._take_ticket()

    def set_mist_enabled(self, enable):
        if not self.connected:
            return None

        if enable:
            self.send_emc_coolant_mist_on(self._tx)
        else:
            self.send_emc_coolant_mist_off(self._tx)
        return self._take_ticket()

    def override_limits(self):
        if not self.connected:
            return None

        self.send_emc_axis_override_limits(self._tx)
        return self._take_ticket()

    def set_adaptive_feed_enabled(self, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.enable = enable

        self.send_emc_motion_adaptive(self._tx)
        return self._take_ticket()

    def set_analog_output(self, index, value):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.index = index
        params.value = value

        self.send_emc_motion_set_aout(self._tx)
        return self._take_ticket()

    def set_block_delete_enabled(self, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.enable = enable

        self.send_emc_task_plan_block_delete(self._tx)
        return self._take_ticket()

    def set_digital_output(self, index, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.index = index
        params.enable = enable

        self.send_emc_motion_set_dout(self._tx)
        return self._take_ticket()

    def set_feed_hold_enabled(self, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.enable = enable

        self.send_emc_traj_set_fh_enable(self._tx)
        return self._take_ticket()

    def set_feed_override_enabled(self, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.enable = enable

        self.send_emc_traj_set_fo_enable(self._tx)
        return self._take_ticket()

    def set_axis_max_position_limit(self, axis, value):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.index = axis
        params.value = value

        self.send_emc_axis_set_max_position_limit(self._tx)
        return self._take_ticket()

    def set_axis_min_position_limit(self, axis, value):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.index = axis
        params.value = value

        self.send_emc_axis_set_min_position_limit(self._tx)
        return self._take_ticket()

    def set_optional_stop_enabled(self, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.enable = enable

        self.send_emc_task_plan_set_optional_stop(self._tx)
        return self._take_ticket()

    def set_spindle_override_enabled(self, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.enable = enable

        self.send_emc_traj_set_so_enable(self._tx)
        return self._take_ticket()

    def set_spindle(self, mode, velocity=0.0):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        if mode == SPINDLE_FORWARD:
            params.velocity = velocity
            self.send_emc_spindle_on(self._tx)
        elif mode == SPINDLE_REVERSE:
            params.velocity = velocity * -1.0
            self.send_emc_spindle_on(self._tx)
        elif mode == SPINDLE_OFF:
            self.send_emc_spindle_off(self._tx)
        elif mode == SPINDLE_INCREASE:
            self.send_emc_spindle_increase(self._tx)
        elif mode == SPINDLE_DECREASE:
            self.send_emc_spindle_decrease(self._tx)
        elif mode == SPINDLE_CONSTANT:
            self.send_emc_spindle_constant(self._tx)
        else:
            self._tx.Clear()
            return None

        return self._take_ticket()

    def set_spindle_override(self, scale):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.scale = scale

        self.send_emc_traj_set_spindle_scale(self._tx)
        return self._take_ticket()

    def set_teleop_enabled(self, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.enable = enable

        self.send_emc_traj_set_teleop_enable(self._tx)
        return self._take_ticket()

    def set_teleop_vector(self, a, b, c, u, v, w):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        pose = params.pose
        pose.a = a
        pose.b = b
        pose.c = c
        pose.u = u
        pose.v = v
        pose.w = w

        self.send_emc_traj_set_teleop_vector(self._tx)
        return self._take_ticket()

    def set_tool_offset(self, index, zoffset, xoffset, diameter, frontangle, backangle, orientation):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        tooldata = params.tool_data
        tooldata.index = index
        tooldata.zoffset = zoffset
        tooldata.xoffset = xoffset
        tooldata.diameter = diameter
        tooldata.frontangle = frontangle
        tooldata.backangle = backangle
        tooldata.orientation = orientation

        self.send_emc_tool_set_offset(self._tx)
        return self._take_ticket()

    def set_trajectory_mode(self, mode):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.traj_mode = mode

        self.send_emc_traj_set_mode(self._tx)
        return self._take_ticket()

    def unhome_axis(self, index):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.index = index

        self.send_emc_axis_unhome(self._tx)
        return self._take_ticket()

    def shutdown(self):
        if not self.connected:
            return None

        self.send_shutdown(self._tx)
        return self._take_ticket()


class ApplicationError(ComponentBase, ErrorBase, ServiceContainer):
    def __init__(self, debug=False):
        ErrorBase.__init__(self, debuglevel=int(debug))
        ComponentBase.__init__(self)
        ServiceContainer.__init__(self)
        self.message_lock = threading.Lock()
        self.connected_condition = threading.Condition(threading.Lock())
        self.debug = debug

        # callbacks
        self.on_connected_changed = []

        self.connected = False
        self.channels = set(['error', 'text', 'display'])
        self.error_list = []

        self._error_service = Service(type_='error')
        self.add_service(self._error_service)
        self.on_services_ready_changed.append(self._on_services_ready_changed)

    def _on_services_ready_changed(self, ready):
        self.error_uri = self._error_service.uri
        self.ready = ready

    def wait_connected(self, timeout=None):
        with self.connected_condition:
            if self.connected:
                return True
            self.connected_condition.wait(timeout=timeout)
            return self.connected

    def emc_nml_error_received(self, _, rx):
        self._error_message_received(rx)

    def emc_nml_text_received(self, _, rx):
        self._error_message_received(rx)

    def emc_nml_display_received(self, _, rx):
        self._error_message_received(rx)

    def emc_operator_text_received(self, _, rx):
        self._error_message_received(rx)

    def emc_operator_display_received(self, _, rx):
        self._error_message_received(rx)

    def emc_operator_error_received(self, _, rx):
        self._error_message_received(rx)

    def _error_message_received(self, rx):
        error = {'type': rx.type, 'notes': []}
        with self.message_lock:
            for note in rx.note:
                error['notes'].append(note)
                self.error_list.append(error)

    # slot
    def update_topics(self):
        self.clear_error_topics()
        for channel in self.channels:
            self.add_error_topic(channel)

    # slot
    def set_connected(self):
        self._update_connected(True)

    # slot
    def clear_connected(self):
        self._update_connected(False)

    def _update_connected(self, connected):
        with self.connected_condition:
            self.connected = connected
            self.connected_condition.notify()
        for cb in self.on_connected_changed:
            cb(connected)

    # returns all received messages and clears the buffer
    def get_messages(self):
        with self.message_lock:
            messages = list(self.error_list)  # make sure to return a copy
            self.error_list = []
            return messages


class ApplicationFile(ComponentBase, ServiceContainer):

    def __init__(self, debug=True):
        self._error_string = ''
        self.on_error_string_changed = []
        ComponentBase.__init__(self)
        ServiceContainer.__init__(self)
        self.debug = debug
        self.state_condition = threading.Condition(threading.Lock())
        self.file_list_lock = threading.Lock()

        self.file_uri = ''
        self.local_file_path = ''
        self.remote_file_path = ''
        self.local_path = ''
        self.remote_path = ''
        self.transfer_state = 'NoTransfer'
        self.bytes_sent = 0.0
        self.bytes_total = 0.0
        self.progress = 0.0
        self.file = None

        self._file_list = []

        self._file_service = Service(type_='file')
        self.add_service(self._file_service)
        self.on_services_ready_changed.append(self._on_services_ready_changed)

    def _on_services_ready_changed(self, ready):
        self.file_uri = self._file_service.uri
        self.ready = ready

    @property
    def error_string(self):
        return self._error_string

    @error_string.setter
    def error_string(self, string):
        if self._error_string is string:
            return
        self._error_string = string
        for cb in self.on_error_string_changed:
            cb(string)

    @property
    def file_list(self):
        with self.file_list_lock:
            return self._file_list

    def _upload_worker(self):
        o = urlparse(self.file_uri)
        # test o.scheme

        filename = os.path.basename(self.local_file_path)
        self.remote_file_path = os.path.join(self.remote_path, filename)

        self._update_state('UploadRunning')  # lets start the upload
        if self.debug:
            print('[file] starting upload of %s' % filename)

        try:
            self.bytes_sent = 0.0
            self.bytes_total = os.path.getsize(self.local_file_path)
            f = open(self.local_file_path, 'r')
        except OSError as e:
            self._update_state('Error')
            self._update_error('file', str(e))
            return

        try:
            self.progress = 0.0
            ftp = ftplib.FTP()
            ftp.connect(host=o.hostname, port=o.port)
            ftp.login()
            ftp.storbinary('STOR %s' % filename, f, blocksize=8192,
                           callback=self._progress_callback)
            ftp.close()
            f.close()
        except Exception as e:
            self._update_state('Error')
            self._update_error('ftp', str(e))
            return

        self._update_state('NoTransfer')  # upload successfully finished
        if self.debug:
            print('[file] upload of %s finished' % filename)

    def _download_worker(self):
        o = urlparse(self.file_uri)
        # test o.scheme

        filename = self.remote_file_path[len(self.remote_path):]  # mid
        self.local_file_path = os.path.join(self.local_path, filename)

        self._update_state('DownloadRunning')  # lets start the upload
        if self.debug:
            print('[file] starting download of %s' % filename)

        try:
            local_path = os.path.dirname(os.path.abspath(self.local_file_path))
            if not os.path.exists(local_path):
                os.makedirs(local_path)
            self.file = open(self.local_file_path, 'w')
        except Exception as e:
            self._update_state('Error')
            self._update_error('file', str(e))
            return

        try:
            ftp = ftplib.FTP()
            ftp.connect(host=o.hostname, port=o.port)
            ftp.login()
            ftp.sendcmd("TYPE i")  # Switch to Binary mode
            self.progress = 0.0
            self.bytes_sent = 0.0
            self.bytes_total = ftp.size(filename)
            ftp.retrbinary('RETR %s' % filename, self._progress_callback)
            ftp.close()
            self.file.close()
            self.file = None
        except Exception as e:
            self._update_state('Error')
            self._update_error('ftp', str(e))
            return

        self._update_state('NoTransfer')  # upload successfully finished
        if self.debug:
            print('[file] download of %s finished' % filename)

    def _refresh_files_worker(self):
        o = urlparse(self.file_uri)
        # test o.scheme

        self._update_state('RefreshRunning')  # lets start the upload
        if self.debug:
            print('[file] starting file list refresh')

        try:
            ftp = ftplib.FTP()
            ftp.connect(host=o.hostname, port=o.port)
            ftp.login()
            with self.file_list_lock:
                self._file_list = ftp.nlst()
            ftp.close()
        except Exception as e:
            self._update_state('Error')
            self._update_error('ftp', str(e))
            return

        self._update_state('NoTransfer')  # upload successfully finished
        if self.debug:
            print('[file] file refresh finished')

    def _remove_file_worker(self, filename):
        o = urlparse(self.file_uri)
        # test o.scheme

        self._update_state('RemoveRunning')  # lets start the upload
        if self.debug:
            print('[file] removing %s' % filename)

        try:
            ftp = ftplib.FTP()
            ftp.connect(host=o.hostname, port=o.port)
            ftp.login()
            ftp.delete(filename)
            ftp.close()
        except Exception as e:
            self._update_state('Error')
            self._update_error('ftp', str(e))
            return

        self._update_state('NoTransfer')  # upload successfully finished
        if self.debug:
            print('[file] removing %s completed' % filename)

    def _progress_callback(self, data):
        if self.file is not None:
            self.file.write(data)
        self.bytes_sent += 8192
        self.progress = self.bytes_sent / self.bytes_total

    def start_upload(self):
        with self.state_condition:
            if not self.ready or self.transfer_state != 'NoTransfer':
                return

        thread = threading.Thread(target=self._upload_worker)
        thread.start()

    def start_download(self):
        with self.state_condition:
            if not self.ready or self.transfer_state != 'NoTransfer':
                return

        thread = threading.Thread(target=self._download_worker)
        thread.start()

    def refresh_files(self):
        with self.state_condition:
            if not self.ready or self.transfer_state != 'NoTransfer':
                return

        thread = threading.Thread(target=self._refresh_files_worker)
        thread.start()

    def remove_file(self, name):
        with self.state_condition:
            if not self.ready or self.transfer_state != 'NoTransfer':
                return

        thread = threading.Thread(target=self._remove_file_worker, args=(name, ))
        thread.start()

    def abort(self):
        raise NotImplementedError('not implemented')

    def wait_completed(self, timeout=None):
        with self.state_condition:
            if self.transfer_state == 'NoTransfer':
                return True
            if self.transfer_state == 'Error':
                return False
            self.state_condition.wait(timeout=timeout)
            return self.transfer_state == 'NoTransfer'

    def _update_state(self, state):
        with self.state_condition:
            if self.transfer_state != state:
                self.transfer_state = state
                self.state_condition.notify()

    def _update_error(self, error, description):
        self.error_string = '[file] error: %s %s' % (error, description)

    def clear_error(self):
        self.error_string = ''

    def start(self):
        pass

    def stop(self):
        pass



from zeroconf import ServiceBrowser, Zeroconf, ServiceInfo


class Service(object):
    def __init__(self, type_=''):
        self.type = type_
        self.domain = 'local'
        self.base_type = 'machinekit'
        self.protocol = 'tcp'
        self.name = ''
        self.uri = ''
        self.uuid = ''
        self.version = 0
        self._ready = False

        self.service_infos = []

        # callback
        self.on_ready_changed = []

    @property
    def ready(self):
        return self._ready

    @ready.setter
    def ready(self, value):
        if value != self._ready:
            self._ready = value
            for cb in self.on_ready_changed:
                cb(value)

    @property
    def typestring(self):
        return '_%s._%s.%s.' % (self.base_type, self.protocol, self.domain)

    def matches_service_info(self, info):
        return self.type == info.properties.get('service') \
            and self.typestring in info.type

    def __eq__(self, other):
        if isinstance(other, ServiceInfo):
            return self.name == other.name
        return False

    def add_service_info(self, info):
        self.service_infos.append(info)
        self._update()

    def remove_service_info(self, info):
        for info in self.service_infos:
            if self == info:
                self.service_infos.remove(info)
                break
        self._update()

    def clear_service_infos(self):
        self.service_infos = []
        self._update()

    def _update(self):
        if len(self.service_infos) > 0:
            info = self.service_infos[0]
            self._set_all_values_from_service_info(info)
            self.ready = True
        else:
            self.ready = False
            self._init_all_values()

    def _set_all_values_from_service_info(self, info):
        self.name = info.name
        self.uri = info.properties.get('dsn', '')
        self.uuid = info.properties.get('uuid', '')
        self.version = info.properties.get('version', '')

    def _init_all_values(self):
        self.name = ''
        self.uri = ''
        self.uuid = ''
        self.version = 0


class ServiceDiscoveryFilter(object):
    def __init__(self, name='', txt_records={}):
        self.name = name
        self.txt_records = txt_records

    def matches_service_info(self, info):
        if not isinstance(info, ServiceInfo):
            raise TypeError('must pass a ServiceInfo object')
        match = True
        if self.name not in info.name:
            match = False
        for name, value in self.txt_records.iteritems():
            if not info.properties[name] == value:
                match = False
                break
        return match


class ServiceDiscovery(object):
    def __init__(self, service_type='machinekit', filter_=ServiceDiscoveryFilter()):
        self.service_type = service_type
        self.filter = filter_

        self.is_ready = False
        self.services = []
        self.browser = None
        self.zeroconf = None

    def _start_discovery(self):
        self.zeroconf = Zeroconf()
        type_string = '_%s._tcp.local.' % self.service_type
        self.browser = ServiceBrowser(self.zeroconf, type_string, self)

    def _stop_discovery(self):
        if self.zeroconf:
            self.zeroconf.close()
            self.zeroconf = None
        self.browser = None
        for service in self.services:
            service.clear_service_infos()

    def remove_service(self, zeroconf, type_, name):
        info = zeroconf.get_service_info(type_, name)
        if info is None:
            return
        for service in self.services:
            if self.filter.matches_service_info(info) and service.matches_service_info(info):
                service.remove_service_info(info)

    def add_service(self, zeroconf, type_, name):
        info = zeroconf.get_service_info(type_, name)
        if info is None:
            return
        for service in self.services:
            if self.filter.matches_service_info(info) and service.matches_service_info(info):
                service.add_service_info(info)

    def _verify_item_and_run(self, item, cmd):
        if isinstance(item, ServiceContainer):
            for service in item.services:
                cmd(service)
        elif isinstance(item, Service):
            cmd(item)
        else:
            raise TypeError('passed unregisterable item')

    def register(self, item):
        if self.is_ready:
            raise RuntimeError('cannot register service when service discovery is already running')
        self._verify_item_and_run(item, self.services.append)

    def unregister(self, item):
        if self.is_ready:
            raise RuntimeError('cannot unregister service when service discovery is already running')
        self._verify_item_and_run(item, self.services.remove)

    def start(self):
        if not self.browser:
            self.is_ready = True
            self._start_discovery()

    def stop(self):
        if self.browser:
            self.is_ready = False
            self._stop_discovery()


class ServiceContainer(object):
    def __init__(self):
        self._services = []
        self._services_ready = False

        self.on_services_ready_changed = []

    @property
    def services(self):
        return self._services

    def add_service(self, service):
        if not isinstance(service, Service):
            raise TypeError('only Service is supported')
        self._services.append(service)
        service.on_ready_changed.append(self._update_services_ready)

    def remove_service(self, service):
        if not isinstance(service, Service):
            raise TypeError('only Service is supported')
        self._services.remove(service)
        service.on_ready_changed.remove(self._update_services_ready)

    @property
    def services_ready(self):
        return self._services_ready

    @services_ready.setter
    def services_ready(self, value):
        if value is not self._services_ready:
            self._services_ready = value
            for cb in self.on_services_ready_changed:
                cb(value)

    def _update_services_ready(self, _):
        ready = True
        for service in self._services:
            if not service.ready:
                ready = False
                break
        self.services_ready = ready



import threading
from dns_sd import ServiceContainer, Service
from common import ComponentBase

# protobuf
from machinetalk.protobuf.message_pb2 import Container
from machinetalk.protobuf.types_pb2 import HAL_FLOAT, HAL_BIT, HAL_S32, HAL_U32, HAL_IN, HAL_OUT, HAL_IO
from machinetalk_core.halremote.remotecomponentbase import RemoteComponentBase


class Pin(object):
    def __init__(self):
        self.name = ''
        self.pintype = HAL_BIT
        self.direction = HAL_IN
        self._synced = False
        self._value = None
        self.handle = 0  # stores handle received on bind
        self.parent = None
        self.synced_condition = threading.Condition(threading.Lock())
        self.value_condition = threading.Condition(threading.Lock())

        # callbacks
        self.on_synced_changed = []
        self.on_value_changed = []

    def wait_synced(self, timeout=None):
        with self.synced_condition:
            if self.synced:
                return True
            self.synced_condition.wait(timeout=timeout)
            return self.synced

    def wait_value(self, timeout=None):
        with self.value_condition:
            if self.value:
                return True
            self.value_condition.wait(timeout=timeout)
            return self.value

    @property
    def value(self):
        with self.value_condition:
            return self._value

    @value.setter
    def value(self, value):
        with self.value_condition:
            if self._value != value:
                self._value = value
                self.value_condition.notify()
                for func in self.on_value_changed:
                    func(value)

    @property
    def synced(self):
        with self.synced_condition:
            return self._synced

    @synced.setter
    def synced(self, value):
        with self.synced_condition:
            if value != self._synced:
                self._synced = value
                self.synced_condition.notify()
                for func in self.on_synced_changed:
                    func(value)

    def set(self, value):
        if self.value != value:
            self.value = value
            self.synced = False
            if self.parent:
                self.parent.pin_change(self)

    def get(self):
        return self.value


class RemoteComponent(ComponentBase, RemoteComponentBase, ServiceContainer):
    def __init__(self, name, debug=False):
        RemoteComponentBase.__init__(self, debuglevel=int(debug))
        ComponentBase.__init__(self)
        ServiceContainer.__init__(self)
        self.connected_condition = threading.Condition(threading.Lock())
        self.debug = debug

        # callbacks
        self.on_connected_changed = []

        self.name = name
        self.pinsbyname = {}
        self.pinsbyhandle = {}
        self.no_create = False
        self.no_bind = False

        self.connected = False

        # more efficient to reuse a protobuf message
        self._tx = Container()

        self._halrcomp_service = Service(type_='halrcomp')
        self._halrcmd_service = Service(type_='halrcmd')
        self.add_service(self._halrcomp_service)
        self.add_service(self._halrcmd_service)
        self.on_services_ready_changed.append(self._on_services_ready_changed)

    def _on_services_ready_changed(self, ready):
        self.halrcomp_uri = self._halrcomp_service.uri
        self.halrcmd_uri = self._halrcmd_service.uri
        self.ready = ready

    def wait_connected(self, timeout=None):
        with self.connected_condition:
            if self.connected:
                return True
            self.connected_condition.wait(timeout=timeout)
            return self.connected

    def set_connected(self):
        with self.connected_condition:
            self.connected = True
            self.connected_condition.notify()
        for cb in self.on_connected_changed:
            cb(self.connected)

    def set_error(self):
        self._check_disconnected()

    def set_disconnected(self):
        self._check_disconnected()

    def set_connecting(self):
        self._check_disconnected()

    def set_timeout(self):
        self._check_disconnected()

    def _check_disconnected(self):
        changed = False
        with self.connected_condition:
            if self.connected:
                self.connected = False
                self.connected_condition.notify()
                changed = True
        if changed:
            for cb in self.on_connected_changed:
                cb(self.connected)

    def halrcomp_incremental_update_received(self, _, rx):
        for rpin in rx.pin:
            lpin = self.pinsbyhandle[rpin.handle]
            self.pin_update(rpin, lpin)

    def halrcomp_full_update_received(self, _, rx):
        if len(rx.comp) == 0:  # empty message
            return

        comp = rx.comp[0]
        for rpin in comp.pin:
            name = rpin.name.split('.')[1]
            lpin = self.pinsbyname[name]
            lpin.handle = rpin.handle
            self.pinsbyhandle[rpin.handle] = lpin
            self.pin_update(rpin, lpin)

        self.pins_synced()  # accept that pins have been synced

    def halrcomp_error_received(self, _, rx):
        pass

    # create a new HAL pin
    def newpin(self, name, pintype, direction):
        pin = Pin()
        pin.name = name
        pin.pintype = pintype
        pin.direction = direction
        pin.parent = self
        self.pinsbyname[name] = pin

        if pintype == HAL_FLOAT:
            pin.value = 0.0
        elif pintype == HAL_BIT:
            pin.value = False
        elif pintype == HAL_S32:
            pin.value = 0
        elif pintype == HAL_U32:
            pin.value = 0

        return pin

    def unsync_pins(self):
        for name in self.pinsbyname:
            self.pinsbyname[name].synced = False

    def getpin(self, name):
        return self.pinsbyname[name]

    def pin_update(self, rpin, lpin):
        if rpin.HasField('halfloat'):
            lpin.value = float(rpin.halfloat)
            lpin.synced = True
        elif rpin.HasField('halbit'):
            lpin.value = bool(rpin.halbit)
            lpin.synced = True
        elif rpin.HasField('hals32'):
            lpin.value = int(rpin.hals32)
            lpin.synced = True
        elif rpin.HasField('halu32'):
            lpin.value = int(rpin.halu32)
            lpin.synced = True

    def pin_change(self, pin):
        if self.debug:
            print('[%s] pin change %s' % (self.name, pin.name))

        if not self.connected:  # accept only when connected
            return
        if pin.direction == HAL_IN:  # only update out and IO pins
            return

        # This message MUST carry a Pin message for each pin which has
        # changed value since the last message of this type.
        # Each Pin message MUST carry the handle field.
        # Each Pin message MAY carry the name field.
        # Each Pin message MUST carry the type field
        # Each Pin message MUST - depending on pin type - carry a halbit,
        # halfloat, hals32, or halu32 field.
        p = self._tx.pin.add()
        p.handle = pin.handle
        p.type = pin.pintype
        if p.type == HAL_FLOAT:
            p.halfloat = float(pin.value)
        elif p.type == HAL_BIT:
            p.halbit = bool(pin.value)
        elif p.type == HAL_S32:
            p.hals32 = int(pin.value)
        elif p.type == HAL_U32:
            p.halu32 = int(pin.value)
        self.send_halrcomp_set(self._tx)

    def bind_component(self):
        c = self._tx.comp.add()
        c.name = self.name
        c.no_create = self.no_create  # for now we create the component
        for name, pin in self.pinsbyname.iteritems():
            p = c.pin.add()
            p.name = '%s.%s' % (self.name, name)
            p.type = pin.pintype
            p.dir = pin.direction
            if p.type == HAL_FLOAT:
                p.halfloat = float(pin.value)
            elif p.type == HAL_BIT:
                p.halbit = bool(pin.value)
            elif p.type == HAL_S32:
                p.hals32 = int(pin.value)
            elif p.type == HAL_U32:
                p.halu32 = int(pin.value)
        if self.debug:
            print('[%s] bind' % self.name)
        self.send_halrcomp_bind(self._tx)

    def add_pins(self):
        self.clear_halrcomp_topics()
        self.add_halrcomp_topic(self.name)

    def remove_pins(self):
        self.pinsbyhandle = {}

    def __getitem__(self, k):
        return self.pinsbyname[k].get()

    def __setitem__(self, k, v):
        self.pinsbyname[k].set(v)


def component(name):
    return RemoteComponent(name)



import zmq
import threading
import uuid
from fysom import Fysom
from ..common.rpcclient import RpcClient
from ..halremote.halrcompsubscribe import HalrcompSubscribe

import machinetalk.protobuf.types_pb2 as pb
from machinetalk.protobuf.message_pb2 import Container


class RemoteConfigBase(object):
    def __init__(self, debuglevel=0, debugname='Remote Config Base'):
        self.debuglevel = debuglevel
        self.debugname = debugname
        self._error_string = ''
        self.on_error_string_changed = []

        # Halrcmd
        self._halrcmd_channel = RpcClient(debuglevel=debuglevel)
        self._halrcmd_channel.debugname = '%s - %s' % (self.debugname, 'halrcmd')
        self._halrcmd_channel.on_state_changed.append(self._halrcmd_channel_state_changed)
        self._halrcmd_channel.on_socket_message_received.append(self._halrcmd_channel_message_received)
        # more efficient to reuse protobuf messages
        self._halrcmd_rx = Container()
        self._halrcmd_tx = Container()

        # Halrcomp
        self._halrcomp_channel = HalrcompSubscribe(debuglevel=debuglevel)
        self._halrcomp_channel.debugname = '%s - %s' % (self.debugname, 'halrcomp')
        self._halrcomp_channel.on_state_changed.append(self._halrcomp_channel_state_changed)
        self._halrcomp_channel.on_socket_message_received.append(self._halrcomp_channel_message_received)
        # more efficient to reuse protobuf messages
        self._halrcomp_rx = Container()

        # callbacks
        self.on_halrcmd_message_received = []
        self.on_halrcomp_message_received = []
        self.on_state_changed = []

        # fsm
        self._fsm = Fysom({'initial': 'down',
                          'events': [
                            {'name': 'connect', 'src': 'down', 'dst': 'trying'},
                            {'name': 'disconnect', 'src': 'trying', 'dst': 'down'},
                            {'name': 'halrcmd_up', 'src': 'trying', 'dst': 'syncing'},
                            {'name': 'halrcmd_trying', 'src': 'syncing', 'dst': 'trying'},
                            {'name': 'halrcomp_up', 'src': 'syncing', 'dst': 'sync'},
                            {'name': 'sync_failed', 'src': 'syncing', 'dst': 'error'},
                            {'name': 'disconnect', 'src': 'syncing', 'dst': 'down'},
                            {'name': 'pins_synced', 'src': 'sync', 'dst': 'synced'},
                            {'name': 'halrcomp_trying', 'src': 'synced', 'dst': 'syncing'},
                            {'name': 'halrcmd_trying', 'src': 'synced', 'dst': 'trying'},
                            {'name': 'set_rejected', 'src': 'synced', 'dst': 'error'},
                            {'name': 'halrcomp_set_msg_sent', 'src': 'synced', 'dst': 'synced'},
                            {'name': 'disconnect', 'src': 'synced', 'dst': 'down'},
                            {'name': 'disconnect', 'src': 'error', 'dst': 'down'},
                          ]})

        self._fsm.ondown = self._on_fsm_down
        self._fsm.onafterconnect = self._on_fsm_connect
        self._fsm.onenterdown = self._on_fsm_down_entry
        self._fsm.onleavedown = self._on_fsm_down_exit
        self._fsm.ontrying = self._on_fsm_trying
        self._fsm.onafterdisconnect = self._on_fsm_disconnect
        self._fsm.onafterhalrcmd_up = self._on_fsm_halrcmd_up
        self._fsm.onsyncing = self._on_fsm_syncing
        self._fsm.onafterhalrcmd_trying = self._on_fsm_halrcmd_trying
        self._fsm.onafterhalrcomp_up = self._on_fsm_halrcomp_up
        self._fsm.onaftersync_failed = self._on_fsm_sync_failed
        self._fsm.onsync = self._on_fsm_sync
        self._fsm.onafterpins_synced = self._on_fsm_pins_synced
        self._fsm.onsynced = self._on_fsm_synced
        self._fsm.onafterhalrcomp_trying = self._on_fsm_halrcomp_trying
        self._fsm.onafterset_rejected = self._on_fsm_set_rejected
        self._fsm.onafterhalrcomp_set_msg_sent = self._on_fsm_halrcomp_set_msg_sent
        self._fsm.onentersynced = self._on_fsm_synced_entry
        self._fsm.onerror = self._on_fsm_error
        self._fsm.onentererror = self._on_fsm_error_entry

    def _on_fsm_down(self, _):
        if self.debuglevel > 0:
            print('[%s]: state DOWN' % self.debugname)
        for cb in self.on_state_changed:
            cb('down')
        return True

    def _on_fsm_connect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event CONNECT' % self.debugname)
        self.add_pins()
        self.start_halrcmd_channel()
        return True

    def _on_fsm_down_entry(self, _):
        if self.debuglevel > 0:
            print('[%s]: state DOWN entry' % self.debugname)
        self.set_disconnected()
        return True

    def _on_fsm_down_exit(self, _):
        if self.debuglevel > 0:
            print('[%s]: state DOWN exit' % self.debugname)
        self.set_connecting()
        return True

    def _on_fsm_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: state TRYING' % self.debugname)
        for cb in self.on_state_changed:
            cb('trying')
        return True

    def _on_fsm_disconnect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event DISCONNECT' % self.debugname)
        self.stop_halrcmd_channel()
        self.stop_halrcomp_channel()
        self.remove_pins()
        return True

    def _on_fsm_halrcmd_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: event HALRCMD UP' % self.debugname)
        return True

    def _on_fsm_syncing(self, _):
        if self.debuglevel > 0:
            print('[%s]: state SYNCING' % self.debugname)
        for cb in self.on_state_changed:
            cb('syncing')
        return True

    def _on_fsm_halrcmd_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: event HALRCMD TRYING' % self.debugname)
        self.stop_halrcomp_channel()
        return True

    def _on_fsm_halrcomp_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: event HALRCOMP UP' % self.debugname)
        return True

    def _on_fsm_sync_failed(self, _):
        if self.debuglevel > 0:
            print('[%s]: event SYNC FAILED' % self.debugname)
        self.stop_halrcomp_channel()
        self.stop_halrcmd_channel()
        return True

    def _on_fsm_sync(self, _):
        if self.debuglevel > 0:
            print('[%s]: state SYNC' % self.debugname)
        for cb in self.on_state_changed:
            cb('sync')
        return True

    def _on_fsm_pins_synced(self, _):
        if self.debuglevel > 0:
            print('[%s]: event PINS SYNCED' % self.debugname)
        return True

    def _on_fsm_synced(self, _):
        if self.debuglevel > 0:
            print('[%s]: state SYNCED' % self.debugname)
        for cb in self.on_state_changed:
            cb('synced')
        return True

    def _on_fsm_halrcomp_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: event HALRCOMP TRYING' % self.debugname)
        self.unsync_pins()
        self.set_timeout()
        return True

    def _on_fsm_set_rejected(self, _):
        if self.debuglevel > 0:
            print('[%s]: event SET REJECTED' % self.debugname)
        self.stop_halrcomp_channel()
        self.stop_halrcmd_channel()
        return True

    def _on_fsm_halrcomp_set_msg_sent(self, _):
        if self.debuglevel > 0:
            print('[%s]: event HALRCOMP SET MSG SENT' % self.debugname)
        return True

    def _on_fsm_synced_entry(self, _):
        if self.debuglevel > 0:
            print('[%s]: state SYNCED entry' % self.debugname)
        self.set_connected()
        return True

    def _on_fsm_error(self, _):
        if self.debuglevel > 0:
            print('[%s]: state ERROR' % self.debugname)
        for cb in self.on_state_changed:
            cb('error')
        return True

    def _on_fsm_error_entry(self, _):
        if self.debuglevel > 0:
            print('[%s]: state ERROR entry' % self.debugname)
        self.set_error()
        return True

    @property
    def error_string(self):
        return self._error_string

    @error_string.setter
    def error_string(self, string):
        if self._error_string is string:
            return
        self._error_string = string
        for cb in self.on_error_string_changed:
            cb(string)

    @property
    def halrcmd_uri(self):
        return self._halrcmd_channel.socket_uri

    @halrcmd_uri.setter
    def halrcmd_uri(self, value):
        self._halrcmd_channel.socket_uri = value

    @property
    def halrcomp_uri(self):
        return self._halrcomp_channel.socket_uri

    @halrcomp_uri.setter
    def halrcomp_uri(self, value):
        self._halrcomp_channel.socket_uri = value

    def add_pins(self):
        print('WARNING: slot add pins unimplemented')

    def remove_pins(self):
        print('WARNING: slot remove pins unimplemented')

    def unsync_pins(self):
        print('WARNING: slot unsync pins unimplemented')

    def set_connected(self):
        print('WARNING: slot set connected unimplemented')

    def set_error(self):
        print('WARNING: slot set error unimplemented')

    def set_disconnected(self):
        print('WARNING: slot set disconnected unimplemented')

    def set_connecting(self):
        print('WARNING: slot set connecting unimplemented')

    def set_timeout(self):
        print('WARNING: slot set timeout unimplemented')

    def pins_synced(self):
        if self._fsm.isstate('sync'):
            self._fsm.pins_synced()

    def start(self):
        if self._fsm.isstate('down'):
            self._fsm.connect()

    def stop(self):
        if self._fsm.isstate('trying'):
            self._fsm.disconnect()
        elif self._fsm.isstate('syncing'):
            self._fsm.disconnect()
        elif self._fsm.isstate('synced'):
            self._fsm.disconnect()
        elif self._fsm.isstate('error'):
            self._fsm.disconnect()

    def add_halrcomp_topic(self, name):
        self._halrcomp_channel.add_socket_topic(name)

    def remove_halrcomp_topic(self, name):
        self._halrcomp_channel.remove_socket_topic(name)

    def clear_halrcomp_topics(self):
        self._halrcomp_channel.clear_socket_topics()

    def start_halrcmd_channel(self):
        self._halrcmd_channel.start()

    def stop_halrcmd_channel(self):
        self._halrcmd_channel.stop()

    def start_halrcomp_channel(self):
        self._halrcomp_channel.start()

    def stop_halrcomp_channel(self):
        self._halrcomp_channel.stop()

    # process all messages received on halrcmd
    def _halrcmd_channel_message_received(self, rx):

        # react to halrcomp set reject message
        if rx.type == pb.MT_HALRCOMP_SET_REJECT:
            # update error string with note
            self.error_string = ''
            for note in rx.note:
                self.error_string += note + '\n'
            if self._fsm.isstate('synced'):
                self._fsm.set_rejected()

        for cb in self.on_halrcmd_message_received:
            cb(rx)

    # process all messages received on halrcomp
    def _halrcomp_channel_message_received(self, identity, rx):

        # react to halrcomp full update message
        if rx.type == pb.MT_HALRCOMP_FULL_UPDATE:
            self.halrcomp_full_update_received(identity, rx)

        # react to halrcomp incremental update message
        elif rx.type == pb.MT_HALRCOMP_INCREMENTAL_UPDATE:
            self.halrcomp_incremental_update_received(identity, rx)

        # react to halrcomp error message
        elif rx.type == pb.MT_HALRCOMP_ERROR:
            # update error string with note
            self.error_string = ''
            for note in rx.note:
                self.error_string += note + '\n'
            if self._fsm.isstate('syncing'):
                self._fsm.sync_failed()
            self.halrcomp_error_received(identity, rx)

        for cb in self.on_halrcomp_message_received:
            cb(identity, rx)

    def halrcomp_full_update_received(self, identity, rx):
        print('SLOT halrcomp full update unimplemented')

    def halrcomp_incremental_update_received(self, identity, rx):
        print('SLOT halrcomp incremental update unimplemented')

    def halrcomp_error_received(self, identity, rx):
        print('SLOT halrcomp error unimplemented')

    def send_halrcmd_message(self, msg_type, tx):
        self._halrcmd_channel.send_socket_message(msg_type, tx)

        if msg_type == pb.MT_HALRCOMP_SET:
            if self._fsm.isstate('synced'):
                self._fsm.halrcomp_set_msg_sent()

    def send_halrcomp_set(self, tx):
        self.send_halrcmd_message(pb.MT_HALRCOMP_SET, tx)

    def _halrcmd_channel_state_changed(self, state):

        if (state == 'trying'):
            if self._fsm.isstate('syncing'):
                self._fsm.halrcmd_trying()
            elif self._fsm.isstate('synced'):
                self._fsm.halrcmd_trying()

        elif (state == 'up'):
            if self._fsm.isstate('trying'):
                self._fsm.halrcmd_up()

    def _halrcomp_channel_state_changed(self, state):

        if (state == 'trying'):
            if self._fsm.isstate('synced'):
                self._fsm.halrcomp_trying()

        elif (state == 'up'):
            if self._fsm.isstate('syncing'):
                self._fsm.halrcomp_up()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ipcmsg.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ipcmsg.proto',
  package='pb',
  serialized_pb=_b('\n\x0cipcmsg.proto\x12\x02pb\"\xa2\x01\n\x07Message\x12\x1d\n\x04type\x18\x01 \x02(\x0e\x32\x0f.pb.MessageType\x12\x0c\n\x04\x61xis\x18\x02 \x01(\x05\x12\x10\n\x08velocity\x18\x03 \x01(\x01\x12\x10\n\x08\x64istance\x18\x04 \x01(\x01\x12\x1d\n\x08jog_type\x18\x05 \x01(\x0e\x32\x0b.pb.JogType\x12\t\n\x01x\x18\x06 \x01(\x01\x12\t\n\x01y\x18\x07 \x01(\x01\x12\x11\n\tconnected\x18\x08 \x01(\x08*?\n\x0bMessageType\x12\x0b\n\x07IPC_JOG\x10\x01\x12\x10\n\x0cIPC_POSITION\x10\x02\x12\x11\n\rIPC_CONNECTED\x10\x03*@\n\x07JogType\x12\x0c\n\x08JOG_STOP\x10\x00\x12\x12\n\x0eJOG_CONTINUOUS\x10\x01\x12\x13\n\x0fJOG_INCREMENTAL\x10\x02')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_MESSAGETYPE = _descriptor.EnumDescriptor(
  name='MessageType',
  full_name='pb.MessageType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='IPC_JOG', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='IPC_POSITION', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='IPC_CONNECTED', index=2, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=185,
  serialized_end=248,
)
_sym_db.RegisterEnumDescriptor(_MESSAGETYPE)

MessageType = enum_type_wrapper.EnumTypeWrapper(_MESSAGETYPE)
_JOGTYPE = _descriptor.EnumDescriptor(
  name='JogType',
  full_name='pb.JogType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='JOG_STOP', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='JOG_CONTINUOUS', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='JOG_INCREMENTAL', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=250,
  serialized_end=314,
)
_sym_db.RegisterEnumDescriptor(_JOGTYPE)

JogType = enum_type_wrapper.EnumTypeWrapper(_JOGTYPE)
IPC_JOG = 1
IPC_POSITION = 2
IPC_CONNECTED = 3
JOG_STOP = 0
JOG_CONTINUOUS = 1
JOG_INCREMENTAL = 2



_MESSAGE = _descriptor.Descriptor(
  name='Message',
  full_name='pb.Message',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='pb.Message.type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='axis', full_name='pb.Message.axis', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='velocity', full_name='pb.Message.velocity', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='distance', full_name='pb.Message.distance', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='jog_type', full_name='pb.Message.jog_type', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='x', full_name='pb.Message.x', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='y', full_name='pb.Message.y', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='connected', full_name='pb.Message.connected', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21,
  serialized_end=183,
)

_MESSAGE.fields_by_name['type'].enum_type = _MESSAGETYPE
_MESSAGE.fields_by_name['jog_type'].enum_type = _JOGTYPE
DESCRIPTOR.message_types_by_name['Message'] = _MESSAGE
DESCRIPTOR.enum_types_by_name['MessageType'] = _MESSAGETYPE
DESCRIPTOR.enum_types_by_name['JogType'] = _JOGTYPE

Message = _reflection.GeneratedProtocolMessageType('Message', (_message.Message,), dict(
  DESCRIPTOR = _MESSAGE,
  __module__ = 'ipcmsg_pb2'
  # @@protoc_insertion_point(class_scope:pb.Message)
  ))
_sym_db.RegisterMessage(Message)


# @@protoc_insertion_point(module_scope)



import zmq
import threading
import uuid
from fysom import Fysom

import machinetalk.protobuf.types_pb2 as pb
from machinetalk.protobuf.message_pb2 import Container


class RpcClient(object):
    def __init__(self, debuglevel=0, debugname='RPC Client'):
        self.debuglevel = debuglevel
        self.debugname = debugname
        self._error_string = ''
        self.on_error_string_changed = []
        # ZeroMQ
        context = zmq.Context()
        context.linger = 0
        self._context = context
        # pipe to signalize a shutdown
        self._shutdown = context.socket(zmq.PUSH)
        self._shutdown_uri = b'inproc://shutdown-%s' % str(uuid.uuid4()).encode()
        self._shutdown.bind(self._shutdown_uri)
        # pipe for outgoing messages
        self._pipe = context.socket(zmq.PUSH)
        self._pipe_uri = b'inproc://pipe-%s' % str(uuid.uuid4()).encode()
        self._pipe.bind(self._pipe_uri)
        self._thread = None  # socket worker tread
        self._tx_lock = threading.Lock()  # lock for outgoing messages

        # Socket
        self.socket_uri = ''
        # more efficient to reuse protobuf messages
        self._socket_rx = Container()
        self._socket_tx = Container()

        # Heartbeat
        self._heartbeat_lock = threading.Lock()
        self._heartbeat_interval = 2500
        self._heartbeat_timer = None
        self._heartbeat_active = False
        self._heartbeat_liveness = 0
        self._heartbeat_reset_liveness = 5

        # callbacks
        self.on_socket_message_received = []
        self.on_state_changed = []

        # fsm
        self._fsm = Fysom({'initial': 'down',
                          'events': [
                            {'name': 'start', 'src': 'down', 'dst': 'trying'},
                            {'name': 'any_msg_received', 'src': 'trying', 'dst': 'up'},
                            {'name': 'heartbeat_timeout', 'src': 'trying', 'dst': 'trying'},
                            {'name': 'heartbeat_tick', 'src': 'trying', 'dst': 'trying'},
                            {'name': 'any_msg_sent', 'src': 'trying', 'dst': 'trying'},
                            {'name': 'stop', 'src': 'trying', 'dst': 'down'},
                            {'name': 'heartbeat_timeout', 'src': 'up', 'dst': 'trying'},
                            {'name': 'heartbeat_tick', 'src': 'up', 'dst': 'up'},
                            {'name': 'any_msg_received', 'src': 'up', 'dst': 'up'},
                            {'name': 'any_msg_sent', 'src': 'up', 'dst': 'up'},
                            {'name': 'stop', 'src': 'up', 'dst': 'down'},
                          ]})

        self._fsm.ondown = self._on_fsm_down
        self._fsm.onafterstart = self._on_fsm_start
        self._fsm.ontrying = self._on_fsm_trying
        self._fsm.onafterany_msg_received = self._on_fsm_any_msg_received
        self._fsm.onafterheartbeat_timeout = self._on_fsm_heartbeat_timeout
        self._fsm.onafterheartbeat_tick = self._on_fsm_heartbeat_tick
        self._fsm.onafterany_msg_sent = self._on_fsm_any_msg_sent
        self._fsm.onafterstop = self._on_fsm_stop
        self._fsm.onup = self._on_fsm_up

    def _on_fsm_down(self, _):
        if self.debuglevel > 0:
            print('[%s]: state DOWN' % self.debugname)
        for cb in self.on_state_changed:
            cb('down')
        return True

    def _on_fsm_start(self, _):
        if self.debuglevel > 0:
            print('[%s]: event START' % self.debugname)
        self.start_socket()
        self.reset_heartbeat_liveness()
        self.send_ping()
        self.start_heartbeat_timer()
        return True

    def _on_fsm_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: state TRYING' % self.debugname)
        for cb in self.on_state_changed:
            cb('trying')
        return True

    def _on_fsm_any_msg_received(self, _):
        if self.debuglevel > 0:
            print('[%s]: event ANY MSG RECEIVED' % self.debugname)
        self.reset_heartbeat_liveness()
        self.reset_heartbeat_timer()
        return True

    def _on_fsm_heartbeat_timeout(self, _):
        if self.debuglevel > 0:
            print('[%s]: event HEARTBEAT TIMEOUT' % self.debugname)
        self.stop_socket()
        self.start_socket()
        self.reset_heartbeat_liveness()
        self.send_ping()
        return True

    def _on_fsm_heartbeat_tick(self, _):
        if self.debuglevel > 0:
            print('[%s]: event HEARTBEAT TICK' % self.debugname)
        self.send_ping()
        return True

    def _on_fsm_any_msg_sent(self, _):
        if self.debuglevel > 0:
            print('[%s]: event ANY MSG SENT' % self.debugname)
        self.reset_heartbeat_timer()
        return True

    def _on_fsm_stop(self, _):
        if self.debuglevel > 0:
            print('[%s]: event STOP' % self.debugname)
        self.stop_heartbeat_timer()
        self.stop_socket()
        return True

    def _on_fsm_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: state UP' % self.debugname)
        for cb in self.on_state_changed:
            cb('up')
        return True

    @property
    def error_string(self):
        return self._error_string

    @error_string.setter
    def error_string(self, string):
        if self._error_string is string:
            return
        self._error_string = string
        for cb in self.on_error_string_changed:
            cb(string)

    def start(self):
        if self._fsm.isstate('down'):
            self._fsm.start()

    def stop(self):
        if self._fsm.isstate('trying'):
            self._fsm.stop()
        elif self._fsm.isstate('up'):
            self._fsm.stop()

    def _socket_worker(self, context, uri):
        poll = zmq.Poller()
        socket = context.socket(zmq.DEALER)
        socket.setsockopt(zmq.LINGER, 0)
        socket.connect(uri)
        poll.register(socket, zmq.POLLIN)

        shutdown = context.socket(zmq.PULL)
        shutdown.connect(self._shutdown_uri)
        poll.register(shutdown, zmq.POLLIN)
        pipe = context.socket(zmq.PULL)
        pipe.connect(self._pipe_uri)
        poll.register(pipe, zmq.POLLIN)

        while True:
            s = dict(poll.poll())
            if shutdown in s:
                shutdown.recv()
                return  # shutdown signal
            if pipe in s:
                socket.send(pipe.recv(), zmq.NOBLOCK)
            if socket in s:
                self._socket_message_received(socket)

    def start_socket(self):
        self._thread = threading.Thread(target=self._socket_worker,
                                        args=(self._context, self.socket_uri,))
        self._thread.start()

    def stop_socket(self):
        self._shutdown.send(' ')  # trigger socket thread shutdown
        self._thread = None

    def _heartbeat_timer_tick(self):
        with self._heartbeat_lock:
            self._heartbeat_timer = None  # timer is dead on tick

        if self.debuglevel > 0:
            print('[%s] heartbeat timer tick' % self.debugname)

        self._heartbeat_liveness -= 1
        if self._heartbeat_liveness == 0:
            if self._fsm.isstate('up'):
                self._fsm.heartbeat_timeout()
            elif self._fsm.isstate('trying'):
                self._fsm.heartbeat_timeout()
            return

        if self._fsm.isstate('up'):
            self._fsm.heartbeat_tick()
        elif self._fsm.isstate('trying'):
            self._fsm.heartbeat_tick()

    def reset_heartbeat_liveness(self):
        self._heartbeat_liveness = self._heartbeat_reset_liveness

    def reset_heartbeat_timer(self):
        if not self._heartbeat_active:
            return

        self._heartbeat_lock.acquire()
        if self._heartbeat_timer:
            self._heartbeat_timer.cancel()
            self._heartbeat_timer = None

        if self._heartbeat_interval > 0:
            self._heartbeat_timer = threading.Timer(self._heartbeat_interval / 1000.0,
                                                 self._heartbeat_timer_tick)
            self._heartbeat_timer.start()
        self._heartbeat_lock.release()
        if self.debuglevel > 0:
            print('[%s] heartbeat timer reset' % self.debugname)

    def start_heartbeat_timer(self):
        self._heartbeat_active = True
        self.reset_heartbeat_timer()

    def stop_heartbeat_timer(self):
        self._heartbeat_active = False
        self._heartbeat_lock.acquire()
        if self._heartbeat_timer:
            self._heartbeat_timer.cancel()
            self._heartbeat_timer = None
        self._heartbeat_lock.release()

    # process all messages received on socket
    def _socket_message_received(self, socket):
        msg = socket.recv()

        try:
            self._socket_rx.ParseFromString(msg)
        except DecodeError as e:
            note = 'Protobuf Decode Error: ' + str(e)
            print(note)  # TODO: decode error
            return

        if self.debuglevel > 0:
            print('[%s] received message' % self.debugname)
            if self.debuglevel > 1:
                print(self.socket_rx)
        rx = self._socket_rx

        # react to any incoming message
        if self._fsm.isstate('trying'):
            self._fsm.any_msg_received()
        elif self._fsm.isstate('up'):
            self._fsm.any_msg_received()

        # react to ping acknowledge message
        if rx.type == pb.MT_PING_ACKNOWLEDGE:
            return   # ping acknowledge is uninteresting

        for cb in self.on_socket_message_received:
            cb(rx)

    def send_socket_message(self, msg_type, tx):
        with self._tx_lock:
            tx.type = msg_type
            if self.debuglevel > 0:
                print('[%s] sending message: %s' % (self.debugname, msg_type))
                if self.debuglevel > 1:
                    print(str(tx))

            self._pipe.send(tx.SerializeToString())
            tx.Clear()

        if self._fsm.isstate('up'):
            self._fsm.any_msg_sent()
        elif self._fsm.isstate('trying'):
            self._fsm.any_msg_sent()

    def send_ping(self):
        tx = self._socket_tx
        self.send_socket_message(pb.MT_PING, tx)



import zmq
import threading
import uuid
from fysom import Fysom
from ..common.simplesubscribe import SimpleSubscribe

import machinetalk.protobuf.types_pb2 as pb
from machinetalk.protobuf.message_pb2 import Container


class LogBase(object):
    def __init__(self, debuglevel=0, debugname='Log Base'):
        self.debuglevel = debuglevel
        self.debugname = debugname
        self._error_string = ''
        self.on_error_string_changed = []

        # Log
        self._log_channel = SimpleSubscribe(debuglevel=debuglevel)
        self._log_channel.debugname = '%s - %s' % (self.debugname, 'log')
        self._log_channel.on_state_changed.append(self._log_channel_state_changed)
        self._log_channel.on_socket_message_received.append(self._log_channel_message_received)
        # more efficient to reuse protobuf messages
        self._log_rx = Container()

        # callbacks
        self.on_log_message_received = []
        self.on_state_changed = []

        # fsm
        self._fsm = Fysom({'initial': 'down',
                          'events': [
                            {'name': 'connect', 'src': 'down', 'dst': 'trying'},
                            {'name': 'log_up', 'src': 'trying', 'dst': 'up'},
                            {'name': 'disconnect', 'src': 'trying', 'dst': 'down'},
                            {'name': 'disconnect', 'src': 'up', 'dst': 'down'},
                          ]})

        self._fsm.ondown = self._on_fsm_down
        self._fsm.onafterconnect = self._on_fsm_connect
        self._fsm.ontrying = self._on_fsm_trying
        self._fsm.onafterlog_up = self._on_fsm_log_up
        self._fsm.onafterdisconnect = self._on_fsm_disconnect
        self._fsm.onup = self._on_fsm_up
        self._fsm.onenterup = self._on_fsm_up_entry
        self._fsm.onleaveup = self._on_fsm_up_exit

    def _on_fsm_down(self, _):
        if self.debuglevel > 0:
            print('[%s]: state DOWN' % self.debugname)
        for cb in self.on_state_changed:
            cb('down')
        return True

    def _on_fsm_connect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event CONNECT' % self.debugname)
        self.update_topics()
        self.start_log_channel()
        return True

    def _on_fsm_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: state TRYING' % self.debugname)
        for cb in self.on_state_changed:
            cb('trying')
        return True

    def _on_fsm_log_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: event LOG UP' % self.debugname)
        return True

    def _on_fsm_disconnect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event DISCONNECT' % self.debugname)
        self.stop_log_channel()
        return True

    def _on_fsm_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: state UP' % self.debugname)
        for cb in self.on_state_changed:
            cb('up')
        return True

    def _on_fsm_up_entry(self, _):
        if self.debuglevel > 0:
            print('[%s]: state UP entry' % self.debugname)
        self.set_connected()
        return True

    def _on_fsm_up_exit(self, _):
        if self.debuglevel > 0:
            print('[%s]: state UP exit' % self.debugname)
        self.clear_connected()
        return True

    @property
    def error_string(self):
        return self._error_string

    @error_string.setter
    def error_string(self, string):
        if self._error_string is string:
            return
        self._error_string = string
        for cb in self.on_error_string_changed:
            cb(string)

    @property
    def log_uri(self):
        return self._log_channel.socket_uri

    @log_uri.setter
    def log_uri(self, value):
        self._log_channel.socket_uri = value

    def update_topics(self):
        print('WARNING: slot update topics unimplemented')

    def set_connected(self):
        print('WARNING: slot set connected unimplemented')

    def clear_connected(self):
        print('WARNING: slot clear connected unimplemented')

    def start(self):
        if self._fsm.isstate('down'):
            self._fsm.connect()

    def stop(self):
        if self._fsm.isstate('trying'):
            self._fsm.disconnect()
        elif self._fsm.isstate('up'):
            self._fsm.disconnect()

    def add_log_topic(self, name):
        self._log_channel.add_socket_topic(name)

    def remove_log_topic(self, name):
        self._log_channel.remove_socket_topic(name)

    def clear_log_topics(self):
        self._log_channel.clear_socket_topics()

    def start_log_channel(self):
        self._log_channel.start()

    def stop_log_channel(self):
        self._log_channel.stop()

    # process all messages received on log
    def _log_channel_message_received(self, identity, rx):

        # react to log message message
        if rx.type == pb.MT_LOG_MESSAGE:
            self.log_message_received(identity, rx)

        for cb in self.on_log_message_received:
            cb(identity, rx)

    def log_message_received(self, identity, rx):
        print('SLOT log message unimplemented')

    def _log_channel_state_changed(self, state):

        if (state == 'up'):
            if self._fsm.isstate('trying'):
                self._fsm.log_up()



#!/usr/bin/env python

from distutils.core import setup
import sys

# We must use setuptools, not distutils, because we need to use the
# namespace_packages option for the "google" package.
try:
    from setuptools import setup, Extension, find_packages
except ImportError:
    try:
        from ez_setup import use_setuptools
        use_setuptools()
        from setuptools import setup, Extension, find_packages
    except ImportError:
        sys.stderr.write(
            "Could not import setuptools; make sure you have setuptools or "
            "ez_setup installed.\n"
        )
        raise

from distutils.command.clean import clean

if sys.version_info[0] == 3:
    # Python 3
    from distutils.command.build_py import build_py_2to3 as build_py
else:
    # Python 2
    from distutils.command.build_py import build_py as build_py
from distutils.spawn import find_executable


if __name__ == '__main__':
    setup(name="pymachinetalk",
          version="0.9.101",
          description="Python bindings for Machinetalk",
          author="Alexander Roessler",
          author_email="alex@machinekoder.com",
          url="https://github.com/machinekit/pymachinetalk",
          namespace_packages=['pymachinetalk'],
          packages=find_packages(),
          install_requires=['setuptools', 'pyzmq', 'zeroconf', 'machinetalk-protobuf', 'fysom'],
          cmdclass={
              'clean': clean,
              'build_py': build_py,
          }
          )



import pytest


@pytest.fixture
def application():
    from pymachinetalk import application
    return application


@pytest.fixture
def dns_sd():
    from pymachinetalk import dns_sd
    return dns_sd


def test_application_integration(application, dns_sd):
    status = application.ApplicationStatus()
    command = application.ApplicationCommand()
    error = application.ApplicationError()
    appfile = application.ApplicationFile()

    sd = dns_sd.ServiceDiscovery()
    sd.register(status)
    sd.register(command)
    sd.register(error)
    sd.register(appfile)
